package slowlogs

import (
	"context"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
)

func TestGetTimingInfo_ValidContext(t *testing.T) {
	ctx := context.Background()
	ctxWithTiming := withTimingInfo(ctx)

	timingInfo := getTimingInfo(ctxWithTiming)
	assert.NotNil(t, timingInfo)
	assert.NotNil(t, timingInfo.MiddlewareTimings)
	assert.NotNil(t, timingInfo.OtherTimings)
	assert.Equal(t, 0, len(timingInfo.MiddlewareTimings))
	assert.Equal(t, 0, len(timingInfo.OtherTimings))
}

func TestGetTimingInfo_InvalidContext(t *testing.T) {
	ctx := context.Background()

	timingInfo := getTimingInfo(ctx)
	assert.NotNil(t, timingInfo)
	assert.NotNil(t, timingInfo.MiddlewareTimings)
	assert.NotNil(t, timingInfo.OtherTimings)
	assert.Equal(t, 0, len(timingInfo.MiddlewareTimings))
	assert.Equal(t, 0, len(timingInfo.OtherTimings))
}

func TestRecorder(t *testing.T) {
	ctx := withTimingInfo(context.Background())
	functionName := "testFunction"

	recorder := Recorder(ctx, functionName)
	time.Sleep(100 * time.Millisecond) // Simulate function execution time
	recorder()

	timingInfo := getTimingInfo(ctx)
	assert.Contains(t, timingInfo.OtherTimings, functionName)

	duration := timingInfo.OtherTimings[functionName]
	assert.GreaterOrEqual(t, duration, 100*time.Millisecond)
}

func TestRecordOtherTiming(t *testing.T) {
	ctx := withTimingInfo(context.Background())
	functionName := "testHandler"
	duration := 200 * time.Millisecond

	recordOtherTiming(ctx, functionName, duration)

	timingInfo := getTimingInfo(ctx)
	assert.Contains(t, timingInfo.OtherTimings, functionName)
	assert.Equal(t, duration, timingInfo.OtherTimings[functionName])
}
